#Program  : PIAGE
#Version  : 1.0
#Date     : 04.03.2008
#Purpose  : Power and sample size estimation for indirect matched case-control association studies of gene-environment interaction
#Programmer: Rebecca Hein, Lars Beckmann
#Contact  : r.hein@dkfz-heidelberg.de
#Copyright (C) 2008 Deutsches Krebsforschungszentrum, DKFZ (German Cancer Research Center), Division of Cancer Epidemiology

#This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation,
#Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#A copy of the GNU Public License is attached to the program documentation.

#Maximization of the expected log-likelihoods L0 and L1 under the null-hypothesis and the alternative and calculation of lambda. 
ML_int_hypothesis<-function(fM,fE,mode_inh,alphaP1,beta,no_LD,hypothesis){

    #aa is the likelihood function for the case. 
    aa<-rep(NA,no_LD)
    #bb is the likelihood function for the control. 
    bb<-rep(NA,no_LD)
    #cc is the penetrance function based on the logistic model, i.e. the probability of beeing diseased given the genotype-exposure combination of each matched pair. 
    cc<-rep(NA,no_LD)
    #multiplier is one of 36 (=2*2*3*3=number of genotype-environment combinations of all case-control pairs) summands which together with the conditional likelihood form the nominator of the expected log-likelihood. 
    multiplierH0<-matrix(NA,no_LD,36)
    #summand is the sum of the products of the penetrance function, and the genotype and exposure frequencies for all 36 combinations of matched pairs and forms the denominator of the expected log-likelihood. 
    summand<-rep(0,no_LD)
    #Maximum likelihood value under H0. 
    max_lik_H0<-rep(NA,no_LD)
    #Maximum likelihood value under H1. 
    max_lik_H1<-rep(NA,no_LD)
    
    #Loop over all LD-values. 
    for(loop in 1:no_LD){
        count<-0
        #Loop over all case-control combinations (3*3*2*2=36). 
        for(case_comb in 1:6){
            #Assignment of genotype, genotypic code, code for the environmental exposure, as well as genotype and exposure frequency to the case. 
            if(case_comb==1){
                gf<-"AA"
                freqGf<-fM^2
                Ef<-1
                freqEf<-fE
                if(mode_inh=="multiplicative"|mode_inh=="log-additive"){
                    Gf<-2
                }else if(mode_inh=="dominant"|mode_inh=="recessive"){
                    Gf<-1
                }
            }else if(case_comb==2){
                gf<-"AA"
                freqGf<-fM^2
                Ef<-0
                freqEf<-1-fE
                if(mode_inh=="multiplicative"|mode_inh=="log-additive"){
                    Gf<-2
                }else if(mode_inh=="dominant"|mode_inh=="recessive"){
                    Gf<-1
                }
            }else if(case_comb==3){
                gf<-"aA"
                freqGf<-2*fM*(1-fM)
                Ef<-1
                freqEf<-fE
                if(mode_inh=="dominant"|mode_inh=="multiplicative"|mode_inh=="log-additive"){
                    Gf<-1
                }else if(mode_inh=="recessive"){
                    Gf<-0
                }
            }else if(case_comb==4){
                gf<-"aA"
                freqGf<-2*fM*(1-fM)
                Ef<-0
                freqEf<-1-fE
                if(mode_inh=="dominant"|mode_inh=="multiplicative"|mode_inh=="log-additive"){
                    Gf<-1
                }else if(mode_inh=="recessive"){
                    Gf<-0
                }
            }else if(case_comb==5){
                gf<-"aa"
                freqGf<-(1-fM)^2
                Gf<-0
                Ef<-1
                freqEf<-fE
            }else if(case_comb==6){
                gf<-"aa"
                freqGf<-(1-fM)^2
                Gf<-0
                Ef<-0
                freqEf<-1-fE
            }
            
            for(ctrl_comb in 1:6){
                count<-count+1
                #Assignment of genotype, genotypic code, code for the environmental exposure, as well as genotype and exposure frequency to the control. 
                if(ctrl_comb==1){
                    gk<-"AA"
                    freqGk<-fM^2
                    Ek<-1
                    freqEk<-fE
                    if(mode_inh=="multiplicative"|mode_inh=="log-additive"){
                        Gk<-2
                    }else if(mode_inh=="dominant"|mode_inh=="recessive"){
                        Gk<-1
                    }
                }else if(ctrl_comb==2){
                    gk<-"AA"
                    freqGk<-fM^2
                    Ek<-0
                    freqEk<-1-fE
                    if(mode_inh=="multiplicative"|mode_inh=="log-additive"){
                        Gk<-2
                    }else if(mode_inh=="dominant"|mode_inh=="recessive"){
                        Gk<-1
                    }
                }else if(ctrl_comb==3){
                    gk<-"aA"
                    freqGk<-2*fM*(1-fM)
                    Ek<-1
                    freqEk<-fE
                    if(mode_inh=="dominant"|mode_inh=="multiplicative"|mode_inh=="log-additive"){
                        Gk<-1
                    }else if(mode_inh=="recessive"){
                        Gk<-0
                    }
                }else if(ctrl_comb==4){
                    gk<-"aA"
                    freqGk<-2*fM*(1-fM)
                    Ek<-0
                    freqEk<-1-fE
                    if(mode_inh=="dominant"|mode_inh=="multiplicative"|mode_inh=="log-additive"){
                        Gk<-1
                    }else if(mode_inh=="recessive"){
                        Gk<-0
                    }
                }else if(ctrl_comb==5){
                    gk<-"aa"
                    freqGk<-(1-fM)^2
                    Gk<-0
                    Ek<-1
                    freqEk<-fE
                }else if(ctrl_comb==6){
                    gk<-"aa"
                    freqGk<-(1-fM)^2
                    Gk<-0
                    Ek<-0
                    freqEk<-1-fE
                }
                
                #aa is the likelihood function for the case. 
                aa[loop]<-exp(alphaP1[loop]+beta[loop,1]*Gf+beta[loop,2]*Ef+beta[loop,3]*Gf*Ef)/(1+exp(alphaP1[loop]+beta[loop,1]*Gf+beta[loop,2]*Ef+beta[loop,3]*Gf*Ef))
                #bb is the likelihood function for the control. 
                bb[loop]<-1-(exp(alphaP1[loop]+beta[loop,1]*Gk+beta[loop,2]*Ek+beta[loop,3]*Gk*Ek)/(1+exp(alphaP1[loop]+beta[loop,1]*Gk+beta[loop,2]*Ek+beta[loop,3]*Gk*Ek)))
                #cc is the penetrance function based on the logistic model, i.e. the probability of beeing diseased given the genotype-exposure combination of each matched pair. 
                cc[loop]<-aa[loop]*bb[loop]
                #multiplier is one of 36 summands which together with the conditional likelihood form the nominator of the expected log-likelihood. 
                multiplierH0[loop,count]<-cc[loop]*freqGf*freqEf*freqGk*freqEk
                #summand is the sum of the products of the penetrance function, and the genotype and exposure frequencies for all 36 combinations of matched pairs and forms the denominator of the expected log-likelihood. 
                summand[loop]<-summand[loop]+(cc[loop]*freqGf*freqEf*freqGk*freqEk)
                
            }#End of ctrl_comb-for-loop. 
        }#End of case_comb-for-loop. 
                          
        #L is the expected log-likelihood. 
        if(mode_inh=="dominant"){        
                L<-function(b1=beta[loop,1],b2=beta[loop,2],b3=beta[loop,3]){
                    -1*((log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(b1+b2+b3))))*multiplierH0[loop,1]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(b1))))*multiplierH0[loop,2]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(b1+b2+b3))))*multiplierH0[loop,3]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(b1))))*multiplierH0[loop,4]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(b2))))*multiplierH0[loop,5]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(0))))*multiplierH0[loop,6]
                    +(log(exp(b1)/(exp(b1)+exp(b1+b2+b3))))*multiplierH0[loop,7]
                    +(log(exp(b1)/(exp(b1)+exp(b1))))*multiplierH0[loop,8]
                    +(log(exp(b1)/(exp(b1)+exp(b1+b2+b3))))*multiplierH0[loop,9]
                    +(log(exp(b1)/(exp(b1)+exp(b1))))*multiplierH0[loop,10]
                    +(log(exp(b1)/(exp(b1)+exp(b2))))*multiplierH0[loop,11]
                    +(log(exp(b1)/(exp(b1)+exp(0))))*multiplierH0[loop,12]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(b1+b2+b3))))*multiplierH0[loop,13]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(b1))))*multiplierH0[loop,14]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(b1+b2+b3))))*multiplierH0[loop,15]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(b1))))*multiplierH0[loop,16]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(b2))))*multiplierH0[loop,17]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(0))))*multiplierH0[loop,18]
                    +(log(exp(b1)/(exp(b1)+exp(b1+b2+b3))))*multiplierH0[loop,19]
                    +(log(exp(b1)/(exp(b1)+exp(b1))))*multiplierH0[loop,20]
                    +(log(exp(b1)/(exp(b1)+exp(b1+b2+b3))))*multiplierH0[loop,21]
                    +(log(exp(b1)/(exp(b1)+exp(b1))))*multiplierH0[loop,22]
                    +(log(exp(b1)/(exp(b1)+exp(b2))))*multiplierH0[loop,23]
                    +(log(exp(b1)/(exp(b1)+exp(0))))*multiplierH0[loop,24]
                    +(log(exp(b2)/(exp(b2)+exp(b1+b2+b3))))*multiplierH0[loop,25]
                    +(log(exp(b2)/(exp(b2)+exp(b1))))*multiplierH0[loop,26]
                    +(log(exp(b2)/(exp(b2)+exp(b1+b2+b3))))*multiplierH0[loop,27]
                    +(log(exp(b2)/(exp(b2)+exp(b1))))*multiplierH0[loop,28]
                    +(log(exp(b2)/(exp(b2)+exp(b2))))*multiplierH0[loop,29]
                    +(log(exp(b2)/(exp(b2)+exp(0))))*multiplierH0[loop,30]
                    +(log(exp(0)/(exp(0)+exp(b1+b2+b3))))*multiplierH0[loop,31]
                    +(log(exp(0)/(exp(0)+exp(b1))))*multiplierH0[loop,32]
                    +(log(exp(0)/(exp(0)+exp(b1+b2+b3))))*multiplierH0[loop,33]
                    +(log(exp(0)/(exp(0)+exp(b1))))*multiplierH0[loop,34]
                    +(log(exp(0)/(exp(0)+exp(b2))))*multiplierH0[loop,35]
                    +(log(exp(0)/(exp(0)+exp(0))))*multiplierH0[loop,36])/summand[loop]
                }
            }else if(mode_inh=="recessive"){
                L<-function(b1=beta[loop,1],b2=beta[loop,2],b3=beta[loop,3]){
                    -1*((log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(b1+b2+b3))))*multiplierH0[loop,1]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(b1))))*multiplierH0[loop,2]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(b2))))*multiplierH0[loop,3]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(0))))*multiplierH0[loop,4]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(b2))))*multiplierH0[loop,5]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(0))))*multiplierH0[loop,6]
                    +(log(exp(b1)/(exp(b1)+exp(b1+b2+b3))))*multiplierH0[loop,7]
                    +(log(exp(b1)/(exp(b1)+exp(b1))))*multiplierH0[loop,8]
                    +(log(exp(b1)/(exp(b1)+exp(b2))))*multiplierH0[loop,9]
                    +(log(exp(b1)/(exp(b1)+exp(0))))*multiplierH0[loop,10]
                    +(log(exp(b1)/(exp(b1)+exp(b2))))*multiplierH0[loop,11]
                    +(log(exp(b1)/(exp(b1)+exp(0))))*multiplierH0[loop,12]
                    +(log(exp(b2)/(exp(b2)+exp(b1+b2+b3))))*multiplierH0[loop,13]
                    +(log(exp(b2)/(exp(b2)+exp(b1))))*multiplierH0[loop,14]
                    +(log(exp(b2)/(exp(b2)+exp(b2))))*multiplierH0[loop,15]
                    +(log(exp(b2)/(exp(b2)+exp(0))))*multiplierH0[loop,16]
                    +(log(exp(b2)/(exp(b2)+exp(b2))))*multiplierH0[loop,17]
                    +(log(exp(b2)/(exp(b2)+exp(0))))*multiplierH0[loop,18]
                    +(log(exp(0)/(exp(0)+exp(b1+b2+b3))))*multiplierH0[loop,19]
                    +(log(exp(0)/(exp(0)+exp(b1))))*multiplierH0[loop,20]
                    +(log(exp(0)/(exp(0)+exp(b2))))*multiplierH0[loop,21]
                    +(log(exp(0)/(exp(0)+exp(0))))*multiplierH0[loop,22]
                    +(log(exp(0)/(exp(0)+exp(b2))))*multiplierH0[loop,23]
                    +(log(exp(0)/(exp(0)+exp(0))))*multiplierH0[loop,24]
                    +(log(exp(b2)/(exp(b2)+exp(b1+b2+b3))))*multiplierH0[loop,25]
                    +(log(exp(b2)/(exp(b2)+exp(b1))))*multiplierH0[loop,26]
                    +(log(exp(b2)/(exp(b2)+exp(b2))))*multiplierH0[loop,27]
                    +(log(exp(b2)/(exp(b2)+exp(0))))*multiplierH0[loop,28]
                    +(log(exp(b2)/(exp(b2)+exp(b2))))*multiplierH0[loop,29]
                    +(log(exp(b2)/(exp(b2)+exp(0))))*multiplierH0[loop,30]
                    +(log(exp(0)/(exp(0)+exp(b1+b2+b3))))*multiplierH0[loop,31]
                    +(log(exp(0)/(exp(0)+exp(b1))))*multiplierH0[loop,32]
                    +(log(exp(0)/(exp(0)+exp(b2))))*multiplierH0[loop,33]
                    +(log(exp(0)/(exp(0)+exp(0))))*multiplierH0[loop,34]
                    +(log(exp(0)/(exp(0)+exp(b2))))*multiplierH0[loop,35]
                    +(log(exp(0)/(exp(0)+exp(0))))*multiplierH0[loop,36])/summand[loop]
                }
            }else if(mode_inh=="multiplicative"|mode_inh=="log-additive"){        
                L<-function(b1=beta[loop,1],b2=beta[loop,2],b3=beta[loop,3]){
                 -1*((log(exp(2*b1+b2+2*b3)/(exp(2*b1+b2+2*b3)+exp(2*b1+b2+2*b3))))*multiplierH0[loop,1]
                    +(log(exp(2*b1+b2+2*b3)/(exp(2*b1+b2+2*b3)+exp(2*b1))))*multiplierH0[loop,2]
                    +(log(exp(2*b1+b2+2*b3)/(exp(2*b1+b2+2*b3)+exp(b1+b2+b3))))*multiplierH0[loop,3]
                    +(log(exp(2*b1+b2+2*b3)/(exp(2*b1+b2+2*b3)+exp(b1))))*multiplierH0[loop,4]
                    +(log(exp(2*b1+b2+2*b3)/(exp(2*b1+b2+2*b3)+exp(b2))))*multiplierH0[loop,5]
                    +(log(exp(2*b1+b2+2*b3)/(exp(2*b1+b2+2*b3)+exp(0))))*multiplierH0[loop,6]
                    +(log(exp(2*b1)/(exp(2*b1)+exp(2*b1+b2+2*b3))))*multiplierH0[loop,7]
                    +(log(exp(2*b1)/(exp(2*b1)+exp(2*b1))))*multiplierH0[loop,8]
                    +(log(exp(2*b1)/(exp(2*b1)+exp(b1+b2+b3))))*multiplierH0[loop,9]
                    +(log(exp(2*b1)/(exp(2*b1)+exp(b1))))*multiplierH0[loop,10]
                    +(log(exp(2*b1)/(exp(2*b1)+exp(b2))))*multiplierH0[loop,11]
                    +(log(exp(2*b1)/(exp(2*b1)+exp(0))))*multiplierH0[loop,12]    
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(2*b1+b2+2*b3))))*multiplierH0[loop,13]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(2*b1))))*multiplierH0[loop,14]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(b1+b2+b3))))*multiplierH0[loop,15]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(b1))))*multiplierH0[loop,16]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(b2))))*multiplierH0[loop,17]
                    +(log(exp(b1+b2+b3)/(exp(b1+b2+b3)+exp(0))))*multiplierH0[loop,18]
                    +(log(exp(b1)/(exp(b1)+exp(2*b1+b2+2*b3))))*multiplierH0[loop,19]
                    +(log(exp(b1)/(exp(b1)+exp(2*b1))))*multiplierH0[loop,20]
                    +(log(exp(b1)/(exp(b1)+exp(b1+b2+b3))))*multiplierH0[loop,21]
                    +(log(exp(b1)/(exp(b1)+exp(b1))))*multiplierH0[loop,22]
                    +(log(exp(b1)/(exp(b1)+exp(b2))))*multiplierH0[loop,23]
                    +(log(exp(b1)/(exp(b1)+exp(0))))*multiplierH0[loop,24]
                    +(log(exp(b2)/(exp(b2)+exp(2*b1+b2+2*b3))))*multiplierH0[loop,25]
                    +(log(exp(b2)/(exp(b2)+exp(2*b1))))*multiplierH0[loop,26]
                    +(log(exp(b2)/(exp(b2)+exp(b1+b2+b3))))*multiplierH0[loop,27]
                    +(log(exp(b2)/(exp(b2)+exp(b1))))*multiplierH0[loop,28]
                    +(log(exp(b2)/(exp(b2)+exp(b2))))*multiplierH0[loop,29]
                    +(log(exp(b2)/(exp(b2)+exp(0))))*multiplierH0[loop,30]
                    +(log(exp(0)/(exp(0)+exp(2*b1+b2+2*b3))))*multiplierH0[loop,31]
                    +(log(exp(0)/(exp(0)+exp(2*b1))))*multiplierH0[loop,32]
                    +(log(exp(0)/(exp(0)+exp(b1+b2+b3))))*multiplierH0[loop,33]
                    +(log(exp(0)/(exp(0)+exp(b1))))*multiplierH0[loop,34]
                    +(log(exp(0)/(exp(0)+exp(b2))))*multiplierH0[loop,35]
                    +(log(exp(0)/(exp(0)+exp(0))))*multiplierH0[loop,36])/summand[loop]  
                }
            }
        if(hypothesis=="GE"){#Test for gene-environment interaction. 
            #Parameter estimation by maximum likelihood method. 
            fit<-mle(L,method="Nelder-Mead",start=list(b1=beta[loop,1],b2=beta[loop,2]),fixed=list(b3=0))
            #Maximum likelihood value under H1. 
            max_lik_H0[loop]<-logLik(fit)
        }else{#G-GE joint test. 
            #Parameter estimation by maximum likelihood method
            fit<-mle(L,method="CG",start=list(b2=beta[loop,2]),fixed=list(b1=0,b3=0))
            #Maximum likelihood value under H1
            max_lik_H0[loop]<-logLik(fit)
        }
        #Parameter estimation by maximum likelihood method
        fit1<-mle(L,method="Nelder-Mead",start=list(b1=beta[loop,1],b2=beta[loop,2],b3=beta[loop,3]))
        
        #Maximum likelihood value under H1
        max_lik_H1[loop]<-logLik(fit1)
    }#End of loop over all LD values
     
    #Calculation of lambda
    lambda<-rep(NA,no_LD)
    lambda<-2*(max_lik_H1-max_lik_H0)
          
    return(lambda)
}
