#Program  : PIAGE
#Version  : 1.0
#Date     : 04.03.2008
#Purpose  : Power and sample size estimation for indirect matched case-control association studies of gene-environment interaction
#Programmer: Rebecca Hein, Lars Beckmann
#Contact  : r.hein@dkfz-heidelberg.de
#Copyright (C) 2008 Deutsches Krebsforschungszentrum, DKFZ (German Cancer Research Center), Division of Cancer Epidemiology

#This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation,
#Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#A copy of the GNU Public License is attached to the program documentation.

#Maximization of the expected log-likelihoods L0 and L1 under the null-hypothesis and the alternative and calculation of lambda. 
ML_mainE<-function(fE,alphaP1,beta){
    
    count<-0
    #multiplier is one of 4 summands which together with the conditional likelihood form the nominator of the expected log-likelihood. 
    multiplier<-rep(NA,4)
    #summand is the sum of the products of the penetrance function and the exposure frequencies for all 4 combinations of matched pairs and forms the denominator of the expected log-likelihood. 
    summand<-0
    
    #Loop over all case-control combinations (2*2=4), 
    for(case_comb in 1:2){
    
        #Assignment of the code for the environmental exposure to the case. 
        #Assignment of the frequency of the environmental exposure. 
        if(case_comb==1){
            Ef<-1
            freqEf<-fE
        }else if(case_comb==2){
            Ef<-0
            freqEf<-1-fE
        }
        
                 
        for(ctrl_comb in 1:2){
            count<-count+1
            #Assignation of code for the environmental exposure to the control. 
            #Assignment of the frequency of the environmental exposure. 
            if(ctrl_comb==1){
                Ek<-1
                freqEk<-fE
            }else if(ctrl_comb==2){
                Ek<-0
                freqEk<-1-fE
            }
            
            #aa is the "likelihood function" for the case. 
            aa<-exp(alphaP1+beta*Ef)/(1+exp(alphaP1+beta*Ef))
            #bb is the "likelihood function" for the control. 
            bb<-1-(exp(alphaP1+beta*Ek)/(1+exp(alphaP1+beta*Ek)))
            #cc is the penetrance function based on the logistic model, i.e. the probability of beeing diseased given the exposure combination of each matched pair. 
            cc<-aa*bb
               
            #multiplier is one of 4 summands which together with the conditional likelihood form the nominator of the expected log-likelihood. 
             multiplier[count]<-cc*freqEf*freqEk
            #summand is the sum of the products of the penetrance function and the exposure frequencies for all 4 combinations of matched pairs and forms the denominator of the expected log-likelihood. 
            summand<-summand+(cc*freqEf*freqEk)
            
        }#End of ctrl_comb-for-loop. 
    }#End of case_comb-for-loop. 
    
    
    #L is the expected log-likelihood. 
    L<-function(b=beta){
        -1*((log(exp(b)/(exp(b)+exp(b)))*multiplier[1]
        +log(exp(b)/(exp(b)+exp(0)))*multiplier[2]
        +log(exp(0)/(exp(0)+exp(b)))*multiplier[3]
        +log(exp(0)/(exp(0)+exp(0)))*multiplier[4])/summand)
    }
        
    #Parameter estimation by maximum likelihood method. 
    fit<-mle(L,start=list(b=beta),method="CG")
    #Maximum likelihood value under H1. 
    max_lik_H1<-logLik(fit)
    
    #Parameter estimation by maximum likelihood method. 
    fit0<-mle(L,fixed=list(b=0),method="CG")
    #Maximum likelihood value under H1. 
    max_lik_H0<-logLik(fit0)
    
    lambda<-2*(max_lik_H1-max_lik_H0)
    
    return(lambda)
}
