#Program  : PIAGE
#Version  : 1.0
#Date     : 04.03.2008
#Purpose  : Power and sample size estimation for indirect matched case-control association studies of gene-environment interaction
#Programmer: Rebecca Hein, Lars Beckmann
#Contact  : r.hein@dkfz-heidelberg.de
#Copyright (C) 2008 Deutsches Krebsforschungszentrum, DKFZ (German Cancer Research Center), Division of Cancer Epidemiology

#This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation,
#Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#A copy of the GNU Public License is attached to the program documentation.

#Estimation of the baseline risk of disease corresponding to the specified hypothesis using a Newton-type algorithm (nlm()). 
#The function to be minimized (fct()) originates from equation (3) in Gauderman, Statistics in Medicine, 2002, 21:35-50. 
P0_est<-function(Kp,or_T,or_E,int_TE,fT,fE,mode_inh,hypothesis){
    alpha_start<-log(Kp/(1-Kp))
    beta1<-log(or_T)
    beta2<-log(or_E)
    ft<-1-fT
    fe<-1-fE
    if(hypothesis=="GE"|hypothesis=="GGE"){
        no_int<-length(int_TE)
        alphaP0<-rep(NA,no_int)
        for(iii in 1:no_int){
            beta3<-log(int_TE[iii])
            if(mode_inh=="dominant"){
                fct<-function(alphaP0,beta1,beta2,beta3,fT,ft,fE,fe,Kp) abs(((exp(alphaP0+beta1+beta2+beta3)/(1+exp(alphaP0+beta1+beta2+beta3))*fT^2*fE+exp(alphaP0+beta1)/(1+exp(alphaP0+beta1))*fT^2*fe+exp(alphaP0+beta1+beta2+beta3)/(1+exp(alphaP0+beta1+beta2+beta3))*2*fT*ft*fE+exp(alphaP0+beta1)/(1+exp(alphaP0+beta1))*2*fT*ft*fe+exp(alphaP0+beta2)/(1+exp(alphaP0+beta2))*ft^2*fE+exp(alphaP0)/(1+exp(alphaP0))*ft^2*fe)-Kp))
            }else if(mode_inh=="recessive"){
                fct<-function(alphaP0,beta1,beta2,beta3,fT,ft,fE,fe,Kp) abs(((exp(alphaP0+beta1+beta2+beta3)/(1+exp(alphaP0+beta1+beta2+beta3))*fT^2*fE+exp(alphaP0+beta1)/(1+exp(alphaP0+beta1))*fT^2*fe+exp(alphaP0+beta2)/(1+exp(alphaP0+beta2))*2*fT*ft*fE+exp(alphaP0)/(1+exp(alphaP0))*2*fT*ft*fe+exp(alphaP0+beta2)/(1+exp(alphaP0+beta2))*ft^2*fE+exp(alphaP0)/(1+exp(alphaP0))*ft^2*fe)-Kp))        
            }else if(mode_inh=="multiplicative"|mode_inh=="log-additive"){
                fct<-function(alphaP0,beta1,beta2,beta3,fT,ft,fE,fe,Kp) abs(((exp(alphaP0+beta1*2+beta2+beta3*2)/(1+exp(alphaP0+beta1*2+beta2+beta3*2))*fT^2*fE+exp(alphaP0+beta1*2)/(1+exp(alphaP0+beta1*2))*fT^2*fe+exp(alphaP0+beta1+beta2+beta3)/(1+exp(alphaP0+beta1+beta2+beta3))*2*fT*ft*fE+exp(alphaP0+beta1)/(1+exp(alphaP0+beta1))*2*fT*ft*fe+exp(alphaP0+beta2)/(1+exp(alphaP0+beta2))*ft^2*fE+exp(alphaP0)/(1+exp(alphaP0))*ft^2*fe)-Kp))
            }
            alphaP0[iii]<-nlm(fct, p=alpha_start,beta1=beta1,beta2=beta2,beta3=beta3,fT=fT,ft=ft,fE=fE,fe=fe,Kp=Kp)$estimate
        }
    }else if(hypothesis=="G"){
        if(mode_inh=="dominant"){
            fct<-function(alphaP0,beta1,fT,ft,Kp) abs(((exp(alphaP0+beta1)/(1+exp(alphaP0+beta1))*fT^2+exp(alphaP0+beta1)/(1+exp(alphaP0+beta1))*2*fT*ft+exp(alphaP0)/(1+exp(alphaP0))*ft^2)-Kp))
        }else if(mode_inh=="recessive"){
            fct<-function(alphaP0,beta1,fT,ft,fE,fe,Kp) abs(((exp(alphaP0+beta1)/(1+exp(alphaP0+beta1))*fT^2+exp(alphaP0)/(1+exp(alphaP0))*2*ft*fT+exp(alphaP0)/(1+exp(alphaP0))*ft^2)-Kp))        
        }else if(mode_inh=="multiplicative"|mode_inh=="log-additive"){
            fct<-function(alphaP0,beta1,fT,ft,fE,fe,Kp) abs(((exp(alphaP0+beta1*2)/(1+exp(alphaP0+beta1*2))*fT^2+exp(alphaP0+beta1)/(1+exp(alphaP0+beta1))*2*fT*ft+exp(alphaP0)/(1+exp(alphaP0))*ft^2)-Kp))
        }
        alphaP0<-nlm(fct, p=alpha_start,beta1=beta1,fT=fT,ft=ft,Kp=Kp)$estimate
    }else if(hypothesis=="E"){
        fct<-function(alphaP0,beta2,fE,fe,Kp) abs(((exp(alphaP0+beta2)/(1+exp(alphaP0+beta2))*fE+exp(alphaP0)/(1+exp(alphaP0))*fe)-Kp))
        alphaP0<-nlm(fct, p=alpha_start,beta2=beta2,fE=fE,fe=fe,Kp=Kp)$estimate
    }
    P0<-exp(alphaP0)/(1+exp(alphaP0))
    return(P0)
}
    
