#Program  : PIAGE
#Version  : 1.0
#Date     : 04.03.2008
#Purpose  : Power and sample size estimation for indirect matched case-control association studies of gene-environment interaction
#Programmer: Rebecca Hein, Lars Beckmann
#Contact  : r.hein@dkfz-heidelberg.de
#Copyright (C) 2008 Deutsches Krebsforschungszentrum, DKFZ (German Cancer Research Center), Division of Cancer Epidemiology

#This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation,
#Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#A copy of the GNU Public License is attached to the program documentation.

PIAGE<-function(hypothesis="GE",fT=0.01,fM_vec=0.01,LD_measure="r2",LD_set=1,fE=0.1,P0=FALSE,Kp=FALSE,or_T=1,or_E=1,int_TE=1,power=FALSE,N=FALSE,mode_inh="dominant",alpha=0.05,test=2,r_path,outputfile,data_path=outputfile){
    
    library(stats4)
    
    if(P0==FALSE){
    #Estimation of the baseline risk of disease if the disease prevalence is specified. 
        par_spec<-"Kp"
        if(Kp>=1||Kp<=0){
            stop("Invalid disease prevalence (Kp) and baseline risk of disease (P0):\t",Kp,", ",P0,"\n")
        }
        P0_file<-paste(r_path,"P0_est.R",sep="")
        source(P0_file)
        P0<-P0_est(Kp,or_T,or_E,int_TE,fT,fE,mode_inh,hypothesis)
    }else{
    #Estimation of the disease prevalence if the baseline risk of disease is specified. 
        par_spec<-"P0"
        if(P0>=1||P0<=0){
            stop("Invalid baseline risk of disease P0:\t",P0,"\n")
        }
        Kp_file<-paste(r_path,"Kp_est.R",sep="")
        source(Kp_file)
        Kp<-Kp_est(P0,or_T,or_E,int_TE,fT,fE,mode_inh,hypothesis)
    }
    
    #Power/sample size estimation for the specified hypothesis. 
    if(hypothesis=="GE"|hypothesis=="GGE"){
        START_file<-paste(r_path,"indirGE.R",sep="")
        source(START_file)
        OUTPUT<-indirGE(fT,or_T,fE,or_E,fM_vec,LD_measure,LD_set,P0,Kp,alpha,int_TE,test,power,N,hypothesis,mode_inh,r_path,data_path,outputfile,par_spec)
    }else if(hypothesis=="G"){
        START_file<-paste(r_path,"indirG.R",sep="")
        source(START_file)
        OUTPUT<-indirG(fT,or_T,fM_vec,LD_measure,LD_set,P0,Kp,alpha,test,power,N,mode_inh,r_path,data_path,outputfile,par_spec)
    }else if(hypothesis=="E"){
        START_file<-paste(r_path,"mainE.R",sep="")
        source(START_file)
        OUTPUT<-mainE(fE,or_E,P0,Kp,alpha,test,power,N,r_path,data_path,outputfile,par_spec)
    }
    if(hypothesis!="G"&hypothesis!="E"&hypothesis!="GE"&hypothesis!="GGE"){
        stop("Invalid hypothesis\t",hypothesis,"\n")                        
    }
    return(OUTPUT)    
}
