#Program  : PIAGE
#Version  : 1.0
#Date     : 04.03.2008
#Purpose  : Power and sample size estimation for indirect matched case-control association studies of gene-environment interaction
#Programmer: Rebecca Hein, Lars Beckmann
#Contact  : r.hein@dkfz-heidelberg.de
#Copyright (C) 2008 Deutsches Krebsforschungszentrum, DKFZ (German Cancer Research Center), Division of Cancer Epidemiology

#This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the
#Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation,
#Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
#A copy of the GNU Public License is attached to the program documentation.

#Estimation of OR_M_bar=Pr(D=1|M=1)*Pr(D=0|M=0)/(Pr(D=0|M=1)*Pr(D=1|M=0))
marginalG<-function(or_M,or_E,int_ME,fE,no_LD,alphaP1){

    beta<-matrix(NA,3,no_LD)
    beta[1,]<-log(or_M)
    beta[2,]<-log(or_E)
    beta[3,]<-log(int_ME)
    
    prob_DG<-rep(0,no_LD)
    prob_dG<-rep(0,no_LD)
    prob_Dg<-rep(0,no_LD)
    prob_dg<-rep(0,no_LD)
    
    for(comb in 1:2){
        #Loop over exposed and unexposed individuals. 
        if(comb==1){
            E<-1
            freqE<-fE
        }else if(comb==2){
            E<-0
            freqE<-1-fE
        }
        
        prob_DG<-prob_DG+(exp(alphaP1+beta[1,]+beta[2,]*E+beta[3,]*E)/(1+exp(alphaP1+beta[1,]+beta[2,]*E+beta[3,]*E))*freqE)
        prob_dG<-prob_dG+(1-(exp(alphaP1+beta[1,]+beta[2,]*E+beta[3,]*E)/(1+exp(alphaP1+beta[1,]+beta[2,]*E+beta[3,]*E))))*freqE
        prob_Dg<-prob_Dg+(exp(alphaP1+beta[2,]*E)/(1+exp(alphaP1+beta[2,]*E))*freqE)
        prob_dg<-prob_dg+(1-(exp(alphaP1+beta[2,]*E)/(1+exp(alphaP1+beta[2,]*E))))*freqE
    }
    
    OR_M_bar<-prob_DG*prob_dg/(prob_dG*prob_Dg)
    
    return(OR_M_bar)
}
